#!/bin/sh

APP_NAME="btcpayserver"

ACTION="${1}"
shift 1

mkdir -p /usr/local/bin

get_image() {
  do_install_detail
}

do_install_detail() {
  local config port IMAGE_NAME username password LAN_IP
  config=$(uci get btcpayserver.@btcpayserver[0].config_path 2>/dev/null)
  port=$(uci get btcpayserver.@btcpayserver[0].port 2>/dev/null)
  IMAGE_NAME=$(uci get btcpayserver.@btcpayserver[0].image_name 2>/dev/null)
  username=$(uci get btcpayserver.@btcpayserver[0].username 2>/dev/null)
  password=$(uci get btcpayserver.@btcpayserver[0].password 2>/dev/null)
  APP_NAME="btcpayserver"

  GEN_PASS=$(< /dev/urandom tr -dc A-Za-z0-9 2>/dev/null | head -c 14; echo)
  GEN_PASS2=$(< /dev/urandom tr -dc A-Za-z0-9 2>/dev/null | head -c 14; echo)

  LAN_IP=$(uci get network.lan.ipaddr)
  LAN_IP="${LAN_IP%/*}"

  if [ -z "$config" ]; then
    echo "config path is empty!"
    exit 1
  fi

  [ -z "$port" ] && port=1880
  [ -z "$IMAGE_NAME" ] && IMAGE_NAME="btcpayserver"

  rm -r /opt/docker2/compose/btcpayserver 2>/dev/null
  mkdir -p /opt/docker2/compose/btcpayserver/

  cat > /opt/docker2/compose/btcpayserver/satoshi.sh <<EOF
git clone https://github.com/btcpayserver/btcpayserver-docker
cd btcpayserver-docker
export BTCPAY_HOST="${LAN_IP}"
export NBITCOIN_NETWORK="${config}"
export BTCPAYGEN_CRYPTO1="btc"
export BTCPAYGEN_ADDITIONAL_FRAGMENTS="opt-save-storage-s"
export BTCPAYGEN_REVERSEPROXY="nginx"
export BTCPAYGEN_LIGHTNING="${IMAGE_NAME}"
export BTCPAY_ENABLE_SSH=true
export NOREVERSEPROXY_HTTP_PORT="${port}"
export BTCPAY_PROTOCOL="http"
export BTCPAYGEN_REVERSEPROXY="none"

. ./btcpay-setup.sh -i
EOF

  bash /opt/docker2/compose/btcpayserver/satoshi.sh

  uci add shortcutmenu lists
  uci set shortcutmenu.@lists[-1].webname="$APP_NAME"
  uci set shortcutmenu.@lists[-1].weburl="$LAN_IP:$port"
  uci set shortcutmenu.@lists[-1].webpath="/"
  uci commit shortcutmenu
}

usage() {
  echo "usage: $0 sub-command"
  echo "where sub-command is one of:"
  echo "      install                Install btcpayserver"
  echo "      upgrade                Upgrade btcpayserver"
  echo "      rm/start/stop/restart  Remove/Start/Stop/Restart btcpayserver"
  echo "      status                 btcpayserver status"
  echo "      port                   btcpayserver port"
}

case "${ACTION}" in
  "install" | "upgrade")
    do_install_detail
    ;;
  "rm")
    echo "Stopping and removing containers..."
    CONTAINER_IDS=$(docker ps -a | grep 'btcpayserver\|generated_' | awk '{print $1}')
    for ID in $CONTAINER_IDS; do
      docker stop "$ID"
      docker rm "$ID"
    done
    IMAGE_IDS=$(docker images | grep 'btcpayserver' | awk '{print $3}')
    if [ -n "$IMAGE_IDS" ]; then
        docker rmi -f $IMAGE_IDS
    fi
    rm -r /opt/docker2/compose/btcpayserver  2>/dev/null
    rm -rf /opt/docker2/compose/btcpayserver/btcpayserver-docker 2>/dev/null  # Removing the btcpayserver-docker directory
    ;;
  "start" | "stop" | "restart")
    APP_NAME="btcpayserver"
    CONTAINER_IDS=$(docker ps -a --filter "ancestor=${APP_NAME}" --format '{{.ID}}')
    for ID in $CONTAINER_IDS; do
      docker "${ACTION}" "${ID}"
    done
    ;;
  "status")
    APP_NAME="btcpayserver"
    CONTAINER_NAMES=$(docker ps -a --filter "name=${APP_NAME}" --format '{{.Names}}')
    docker ps --all -f "name=${CONTAINER_NAMES}" --format '{{.Status}}'
    ;;
  "port")
    APP_NAME="btcpayserver"
    CONTAINER_NAMES=$(docker ps -a --filter "ancestor=${APP_NAME}" --format '{{.Names}}')
    docker ps --all -f "name=${CONTAINER_NAMES}" --format '{{.Ports}}' | grep -o -m1 '0.0.0.0:[0-9]*' | sed 's/0.0.0.0://'
    ;;
  *)
    usage
    exit 1
    ;;
esac